using System;
using System.Collections;
using System.Data;
using System.Configuration;
using System.Net;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Text;

namespace ASPItalia.Books.Web.Chapter11
{

    public class FtpDataSource : HierarchicalDataSourceControl, IDataSource
    {
        private string[] _viewNames;

        private Uri _host;
        private IWebProxy _proxy;
        private ICredentials _credentials;
        private bool _usePassive;
        private string _initialPath;

        private FtpParser _parser = new FtpParser(Encoding.ASCII);

        #region Public methods

        public bool MakeDirectory(string directoryName)
        {
            FtpWebRequest request = GetFtpRequest(directoryName);
            request.Method = WebRequestMethods.Ftp.MakeDirectory;

            using (FtpWebResponse response = (FtpWebResponse)request.GetResponse())
            {
                return (response.StatusCode == FtpStatusCode.PathnameCreated);
            }

        }

        #endregion

        protected override HierarchicalDataSourceView GetHierarchicalView(string viewPath)
        {
            return new FtpHierarchicalDataSourceView(this, viewPath);
        }

        internal FtpWebRequest GetFtpRequest(string path)
        {
            if (this.Host == null)
                throw new HttpException(String.Format("Please specified a Host for FtpDataSource {0}", this.ID));
            
            UriBuilder uri = new UriBuilder(this.Host);
            if (!String.IsNullOrEmpty(this.InitialPath))
                uri.Path = VirtualPathUtility.Combine(uri.Path, this.InitialPath);
            if (!String.IsNullOrEmpty(path))
                uri.Path += path;
            
            // Creo l'oggetto ftp
            FtpWebRequest ftp = (FtpWebRequest)FtpWebRequest.Create(uri.Uri);
            
            // Setto le varie impostazioni
            if (this.Credentials != null)
                ftp.Credentials = this.Credentials;
            if (this.Proxy != null)
                ftp.Proxy = this.Proxy;
            ftp.UsePassive = this.UsePassive;
            
            return ftp;
        }

        #region Properties

        /// <summary>
        /// Indica se mostrare in modo gerarchico anche i files. Default false
        /// </summary>
        public bool ShowHierarchicalFiles
        {
            get
            {
                object o = ViewState["ShowHierarchicalFiles"];
                if (o != null)
                    return (bool)o;
                else
                    return false;
            }
            set
            {
                ViewState["ShowHierarchicalFiles"] = true;

                this.OnDataSourceChanged(EventArgs.Empty);
            }
        }

        /// <summary>
        /// Indica il path iniziale sul FTP
        /// </summary>
        public String InitialPath
        {
            get
            {
                if (_initialPath == null)
                    _initialPath = String.Empty;
                return _initialPath;
            }
            set
            {
                if (this.InitialPath != value)
                {
                    _initialPath = value;
                    this.OnDataSourceChanged(EventArgs.Empty);
                }
            }
        }

        /// <summary>
        /// Uri del host ftp
        /// </summary>
        public Uri Host
        {
            get { return _host; }
            set
            {
                if (this.Host != value)
                {
                    _host = value;
                    this.OnDataSourceChanged(EventArgs.Empty);
                }
            }
        }

        /// <summary>
        /// Eventuale proxy da utilizzare
        /// </summary>
        public IWebProxy Proxy
        {
            get { return _proxy; }
            set
            {
                if (this.Proxy != value)
                {
                    _proxy = value;
                    this.OnDataSourceChanged(EventArgs.Empty);
                }
            }
        }

        /// <summary>
        /// Credenziali di accesso
        /// </summary>
        public ICredentials Credentials
        {
            get { return _credentials; }
            set
            {
                if (this.Credentials != value)
                {
                    _credentials = value;
                    this.OnDataSourceChanged(EventArgs.Empty);
                }
            }
        }

        /// <summary>
        /// Modalit passiva
        /// </summary>
        public bool UsePassive
        {
            get { return _usePassive; }
            set
            {
                if (this.UsePassive != value)
                {
                    _usePassive = value;
                    this.OnDataSourceChanged(EventArgs.Empty);
                }
            }
        }

        internal FtpParser Parser
        {
            get { return this._parser; }
        }

        #endregion

        #region IDataSource implement

        event EventHandler IDataSource.DataSourceChanged
        {
            add { ((IHierarchicalDataSource) this).DataSourceChanged += value; }
            remove { ((IHierarchicalDataSource)this).DataSourceChanged -= value; }
        }

        DataSourceView IDataSource.GetView(string viewName)
        {
            if (viewName.Length == 0)
            {
                viewName = "DefaultView";
            }
            return new FtpDataSourceView(this, viewName);
        }

        ICollection IDataSource.GetViewNames()
        {
            if (this._viewNames == null)
            {
                this._viewNames = new string[] { "DefaultView" };
            }
            return this._viewNames;
        }

        #endregion
    }

}